/* utils/sxcl-filter.C: part of meabench, an MEA recording and analysis tool
** Copyright (C) 2000-2002  Daniel Wagenaar (wagenaar@caltech.edu)
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// sxcl-filter.C

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <math.h>
#include <common/Config.H>

void usage() {
  fprintf(stderr,"Usage: sxcl-filter bins thresh\n"
"\n"
"Reads sxclog output from stdin, and outputs a list of all (c0,c1,dt) tuples\n"
"that have a significantly enhanced count (more than THRESH std devs over\n"
"the Poisson expectation value).\n"
"BINS must be the number of bins generated by sxclog, or trash will be produced.\n"
"\n"
"Output is: classnr from_hw to_hw bin bin_start_ms bin_end_ms count\n"
"                                                           expected overcount\n"
"\n"
"where: CLASSNR is a sequentially assigned integer to refer to this bin\n"
"               elsewhere\n"
"       COUNT is the number of observations in this bin\n"
"       EXPECTED is the number expected by Poisson statistics\n"
"       OVERCOUNT is the number of std.devs by which COUNT exceeds EXPECTED.\n"
"");
  exit(1);
}

int main(int argc, char **argv) {
  if (argc!=3)
    usage();
  int nbins = atoi(argv[1]);
  float thresh=atof(argv[2]);

  int *count[TOTALCHANS][TOTALCHANS];
  for (int c=0; c<TOTALCHANS; c++)
    for (int d=0; d<TOTALCHANS; d++) {
      count[c][d] = new int[nbins];
      for (int i=0; i<nbins; i++)
	scanf("%i",&count[c][d][i]);
    }

  double t0=0;
  double st[nbins];
  double et[nbins];
  double dt[nbins];
  for (int i=0; i<nbins; i++) {
    double t1;
    scanf("%lg",&t1);
    dt[i]=t1-t0;
    st[i]=t0;
    et[i]=t1;
    //    fprintf(stderr,"%i:%g-%g\n",i,t0,t1);
    t0=t1;
  }


  
  double ttime;
  scanf("%lg",&ttime);

  //  fprintf(stderr,"ttime = %g\n",ttime);
  
  int totals[TOTALCHANS];
  for (int c=0; c<TOTALCHANS; c++)
    scanf("%i",&totals[c]);

  //  fprintf(stderr,"Totals are:\n");
  int tottot=0;
  for (int c=0; c<TOTALCHANS; c++) {
    fprintf(stderr,"%i ",totals[c]);
    if (c%8==7)
      fprintf(stderr,"\n");
    tottot+=totals[c];
  }
  //  fprintf(stderr," total total: %i (%i)\n",tottot,tottot*164);

  int classnr=0;
  for (int c=0; c<TOTALCHANS; c++)
    for (int d=0; d<TOTALCHANS; d++)
      for (int i=0; i<nbins; i++) {
	double observ = count[c][d][i];
	double expect = totals[c]*totals[d]*dt[i]/ttime;
	if (observ > expect + sqrt(expect)*thresh)
	  printf("%i %i %i %i %g %g %g %g %g\n",
		 classnr++,
		 c,d,i,st[i],et[i],
		 observ,expect,
		 (observ-expect)/sqrt(expect));
      }
  return 0;
}
